#include <assert.h>
#include <stdio.h>
#include <stdlib.h>

#include <yubihsm.h>

#define DEFAULT_CONNECTOR_URL "http://127.0.0.1:12345"

int main(void) {
  yh_connector *connector = NULL;
  yh_rc yrc = YHR_GENERIC_ERROR;

  yrc = yh_init();
  if (yrc != YHR_SUCCESS) {
    fprintf(stderr, "unable to initialize yubihsm: %s\n", yh_strerror(yrc));
    exit(EXIT_FAILURE);
  }

  yrc = yh_init_connector(DEFAULT_CONNECTOR_URL, &connector);
  assert(yrc == YHR_SUCCESS);

  yrc = yh_connect_best(&connector, 1, NULL);
  assert(yrc == YHR_SUCCESS);

  char *received_url;
  yrc = yh_get_connector_address(connector, &received_url);
  assert(yrc == YHR_SUCCESS);

  printf("Successfully connected to %s, device is ", received_url);
  if (yh_connector_has_device(connector) == false) {
    printf("not present\n");
    exit(EXIT_FAILURE); // This won't happen since we manged to connect
  }
  printf("present\n");

  uint8_t c_major, c_minor, c_patch;
  yrc = yh_get_connector_version(connector, &c_major, &c_minor, &c_patch);
  assert(yrc == YHR_SUCCESS);
  printf("Connector Version: %hhu.%hhu.%hhu\n", c_major, c_minor, c_patch);

  uint8_t d_major, d_minor, d_patch;
  uint32_t serial;
  uint8_t log_total, log_used;
  yh_algorithm algorithms[YH_MAX_ALGORITHM_COUNT];
  size_t n_algorithms = sizeof(algorithms);
  yrc =
    yh_util_get_device_info(connector, &d_major, &d_minor, &d_patch, &serial,
                            &log_total, &log_used, algorithms, &n_algorithms);
  assert(yrc == YHR_SUCCESS);

  printf("Device Version: %hhu.%hhu.%hhu\n", d_major, d_minor, d_patch);
  printf("Serial: %d\n", serial);
  printf("Log: %d/%d (used/total)\n", log_used, log_total);
  printf("Supported algorithms:\n");
  for (size_t i = 0; i < n_algorithms; i++) {
    const char *str;
    yh_algo_to_string(algorithms[i], &str);
    printf("%s\n", str);
  }

  yrc = yh_exit();
  assert(yrc == YHR_SUCCESS);

  return 0;
}
